@@ -126,7 +126,7 @@ static int info(void)
 	}
 	err = snd_mixer_load(mhandle);
 	if (err < 0) {
-		error("Mixer load error: %s", card, snd_strerror(err));
+		error("Mixer load %s error: %s", card, snd_strerror(err));
 		snd_mixer_close(mhandle);
 		return err;
 	}
@@ -774,7 +774,7 @@ static int selems(int level)
 	}
 	err = snd_mixer_load(handle);
 	if (err < 0) {
-		error("Mixer load error: %s", card, snd_strerror(err));
+		error("Mixer %s load error: %s", card, snd_strerror(err));
 		snd_mixer_close(handle);
 		return err;
 	}
@@ -1109,7 +1109,7 @@ static int sset(unsigned int argc, char *argv[], int roflag)
 	int err;
 	unsigned int idx;
 	snd_mixer_selem_channel_id_t chn;
-	unsigned int channels = ~0UL;
+	unsigned int channels = ~0U;
 	unsigned int dir = 3;
 	long pmin, pmax, cmin, cmax;
 	snd_mixer_t *handle;
@@ -1145,7 +1145,7 @@ static int sset(unsigned int argc, char *argv[], int roflag)
 	}
 	err = snd_mixer_load(handle);
 	if (err < 0) {
-		error("Mixer load error: %s", card, snd_strerror(err));
+		error("Mixer %s load error: %s", card, snd_strerror(err));
 		snd_mixer_close(handle);
 		return err;
 	}
@@ -1427,7 +1427,7 @@ static int sevents(int argc ATTRIBUTE_UNUSED, char *argv[] ATTRIBUTE_UNUSED)
 	snd_mixer_set_callback(handle, mixer_event);
 	err = snd_mixer_load(handle);
 	if (err < 0) {
-		error("Mixer load error: %s", card, snd_strerror(err));
+		error("Mixer %s load error: %s", card, snd_strerror(err));
 		snd_mixer_close(handle);
 		return err;
 	}
@@ -1486,7 +1486,7 @@ int main(int argc, char *argv[])
 			break;
 		case 'D':
 		case HELPID_DEVICE:
-			strncpy(card, optarg, sizeof(card-1));
+			strncpy(card, optarg, sizeof(card)-1);
 			card[sizeof(card)-1] = '\0';
 			break;
 		case 'q':
